@@ -13,7 +13,7 @@
 ##
 ##     Sukumaran, J. and M. T. Holder. 2010. DendroPy: a Python library
 ##     for phylogenetic computing. Bioinformatics 26: 1569-1571.
-##
+#
 ##############################################################################
 
 """
@@ -22,12 +22,9 @@ Parsing of NEWICK-format tree from a stream.
 
 import re
 import warnings
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
 from dendropy.utility import error
 from dendropy.utility import deprecate
+from dendropy.utility.textprocessing import StringIO
 from dendropy.dataio import tokenizer
 from dendropy.dataio import nexusprocessing
 from dendropy.dataio import ioservice
@@ -121,8 +118,7 @@ class NewickReader(ioservice.DataReader):
                     stream=stream)
 
     def __init__(self, **kwargs):
-        """
-        Keyword Arguments
+        """Keyword Arguments
         -----------------
         rooting : string, {['default-unrooted'], 'default-rooted', 'force-unrooted', 'force-rooted'}
             Specifies how trees in the data source should be intepreted with
@@ -143,51 +139,58 @@ class NewickReader(ioservice.DataReader):
             Specifies the type of the edge lengths (``int`` or ``float``). Tokens
             interpreted as branch lengths will be cast to this type.
             Defaults to ``float``.
-        suppress_edge_lengths : boolean, default: `False`
-            If `True`, edge length values will not be processed. If `False`,
+        suppress_edge_lengths : boolean, default: |False|
+            If |True|, edge length values will not be processed. If |False|,
             edge length values will be processed.
-        extract_comment_metadata : boolean, default: `True`
-            If `True` (default), any comments that begin with '&' or '&&' will
+        extract_comment_metadata : boolean, default: |True|
+            If |True| (default), any comments that begin with '&' or '&&' will
             be parsed and stored as part of the annotation set of the
             corresponding object (accessible through the ``annotations``
             attribute of the object). This requires that the comment
             contents conform to a particular format (NHX or BEAST: 'field =
-            value'). If `False`, then the comments will not be parsed,
+            value'). If |False|, then the comments will not be parsed,
             but will be instead stored directly as elements of the ``comments``
             list attribute of the associated object.
-        store_tree_weights : boolean, default: `False`
-            If `True`, process the tree weight (e.g. "[&W 1/2]") comment
-            associated with each tree, if any. Defaults to `False`.
-        encode_splits : boolean, default: `False`
-            If `True`, split hash bitmasks will be calculated and attached to
-            the edges.
-        finish_node_fn : function object, default: `None`
+        store_tree_weights : boolean, default: |False|
+            If |True|, process the tree weight (e.g. "[&W 1/2]") comment
+            associated with each tree, if any. Defaults to |False|.
+        finish_node_fn : function object, default: |None|
             If specified, this function will be applied to each node after
             it has been constructed.
-        case_sensitive_taxon_labels : boolean, default: `False`
-            If `True`, then taxon labels are case sensitive (e.g., "P.regius"
+        case_sensitive_taxon_labels : boolean, default: |False|
+            If |True|, then taxon labels are case sensitive (e.g., "P.regius"
             and "P.REGIUS" wil be treated as different operation taxonomic
             unit concepts). Otherwise, taxon label intepretation will be made
             without regard for case.
-        preserve_underscores : boolean, default: `False`
-            If `True`, unquoted underscores in labels will *not* converted to
-            spaces. Defaults to `False`: all underscores not protected by
+        preserve_underscores : boolean, default: |False|
+            If |True|, unquoted underscores in labels will *not* converted to
+            spaces. Defaults to |False|: all underscores not protected by
             quotes will be converted to spaces.
-        suppress_internal_node_taxa : boolean, default: `True`
-            If `False`, internal node labels will be instantantiated into
-            |Taxon| objects. If `True`, internal node labels
+        suppress_internal_node_taxa : boolean, default: |True|
+            If |False|, internal node labels will be instantantiated into
+            |Taxon| objects. If |True|, internal node labels
             will *not* be instantantiated as strings.
-        suppress_leaf_node_taxa : boolean, default: `False`
-            If `False`, leaf (external) node labels will be instantantiated
-            into |Taxon| objects. If `True`, leaff (external) node
+        suppress_leaf_node_taxa : boolean, default: |False|
+            If |False|, leaf (external) node labels will be instantantiated
+            into |Taxon| objects. If |True|, leaff (external) node
             labels will *not* be instantantiated as strings.
-        terminating_semicolon_required : boolean, default: `True`
-            If `True` [default], then a tree statement that does not end in a
-            semi-colon is an error. If `False`, then no error will be raised.
-        ignore_unrecognized_keyword_arguments : boolean, default: `False`
-            If `True`, then unsupported or unrecognized keyword arguments will
-            not result in an error. Default is `False`: unsupported keyword
+        is_parse_jplace_tokens : boolean: |False|
+            If |True|, then accept edge numbering according to the jplace
+            format, as described in Matsen et. al. PLoS One, 2012
+            http://dx.doi.org/10.1371/journal.pone.0031009. An instance variable
+            edge_index is added to the returned tree, and an edge_number is
+            added to each edge. If False [default], encountering edge labels
+            raises a NewickReaderMalformedStatementError.
+        is_assign_internal_labels_to_edges : boolean, default: |None|
+            If |True|, internal node labels will be assigned as edge labels.
+        terminating_semicolon_required : boolean, default: |True|
+            If |True| [default], then a tree statement that does not end in a
+            semi-colon is an error. If |False|, then no error will be raised.
+        ignore_unrecognized_keyword_arguments : boolean, default: |False|
+            If |True|, then unsupported or unrecognized keyword arguments will
+            not result in an error. Default is |False|: unsupported keyword
             arguments will result in an error.
+
         """
 
         # base
@@ -256,6 +259,10 @@ class NewickReader(ioservice.DataReader):
         self.suppress_internal_node_taxa = kwargs.pop("suppress_internal_node_taxa", True)
         self.suppress_leaf_node_taxa = kwargs.pop("suppress_external_node_taxa", False) # legacy (will be deprecated)
         self.suppress_leaf_node_taxa = kwargs.pop("suppress_leaf_node_taxa", self.suppress_leaf_node_taxa)
+        self.is_parse_jplace_tokens = kwargs.pop("is_parse_jplace_tokens", False)
+        self.is_assign_internal_labels_to_edges = kwargs.pop("is_assign_internal_labels_to_edges", None)
+        if self.is_assign_internal_labels_to_edges and not self.suppress_internal_node_taxa:
+            raise ValueError("Conflicting options: cannot simultaneously assign internal labels to edges and to internal taxa")
         self.terminating_semicolon_required = kwargs.pop("terminating_semicolon_required", True)
         self.check_for_unused_keyword_arguments(kwargs)
 
@@ -558,7 +565,7 @@ class NewickReader(ioservice.DataReader):
         self._tree_statement_complete = False
         if is_internal_node is None:
             # Initial call using ``seed_node`` does not set ``is_internal_node`` to
-            # `True` or `False`, explicitly, but rather `None`. If this is the
+            # |True| or |False|, explicitly, but rather |None|. If this is the
             # case, the rest of the tree has be constructed, and we simply look
             # at whether there are children or not to determine if it is an
             # internal node. This approach allows for a single-tip tree.
@@ -585,12 +592,16 @@ class NewickReader(ioservice.DataReader):
                 try:
                     nexus_tokenizer.require_next_token()
                 except tokenizer.Tokenizer.UnexpectedEndOfStreamError as e:
-                    message = e.message + ". (Perhaps the terminating semicolon for the tree statement is missing? If so, add a semicolon to the tree statement or specify 'terminating_semicolon_required=False' to allow for missing semicolons)"
-                    raise tokenizer.Tokenizer.UnexpectedEndOfStreamError(
-                            message=message,
-                            line_num=e.line_num,
-                            col_num=e.col_num,
-                            stream=e.stream)
+                    if self.terminating_semicolon_required:
+                        message = e.message + ". (Perhaps the terminating semicolon for the tree statement is missing? If so, add a semicolon to the tree statement or specify 'terminating_semicolon_required=False' to allow for missing semicolons)"
+                        raise tokenizer.Tokenizer.UnexpectedEndOfStreamError(
+                                message=message,
+                                line_num=e.line_num,
+                                col_num=e.col_num,
+                                stream=e.stream)
+                    else:
+                        self._tree_statement_complete = True
+                        break
 
             elif nexus_tokenizer.current_token == ")": #253
                 # closing of parent token
@@ -621,11 +632,28 @@ class NewickReader(ioservice.DataReader):
                         line_num=nexus_tokenizer.token_line_num,
                         col_num=nexus_tokenizer.token_column_num,
                         stream=nexus_tokenizer.src)
+            elif self.is_parse_jplace_tokens and nexus_tokenizer.current_token == '{':
+                # Edge number from .jplace format
+                nexus_tokenizer.require_next_token()
+                edge_number = int(nexus_tokenizer.current_token)
+                edge = current_node.edge
+                edge.edge_number = edge_number
+                try:
+                    tree.edge_index.insert(edge_number, edge)
+                except AttributeError:
+                    tree.edge_index = []
+                    tree.edge_index.insert(edge_number, edge)
+                nexus_tokenizer.require_next_token() # for closing '}'
+                nexus_tokenizer.require_next_token()
             else: #267
                 # label
                 if label_parsed: #269
+                    msg = "Expecting ':'"
+                    if self.is_parse_jplace_tokens:
+                        msg += ", '{'"
+                    msg += ", ')', ',' or ';' after reading label but found '{}'".format(nexus_tokenizer.current_token)
                     raise NewickReader.NewickReaderMalformedStatementError(
-                            message="Expecting ':', ')', ',' or ';' after reading label but found '{}'".format(nexus_tokenizer.current_token),
+                            message=msg,
                             line_num=nexus_tokenizer.token_line_num,
                             col_num=nexus_tokenizer.token_column_num,
                             stream=nexus_tokenizer.src)
@@ -634,7 +662,10 @@ class NewickReader(ioservice.DataReader):
                     label = nexus_tokenizer.current_token
                     if ( (is_internal_node and self.suppress_internal_node_taxa)
                             or ((not is_internal_node) and self.suppress_leaf_node_taxa) ):
-                        current_node.label = label
+                        if self.is_assign_internal_labels_to_edges:
+                            current_node.edge.label = label
+                        else:
+                            current_node.label = label
                     else:
                         node_taxon = taxon_symbol_map_fn(label)
                         if node_taxon in self._seen_taxa:
