@@ -21,12 +21,9 @@ Writing data in NEXUS format to an output stream.
 """
 
 import re
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
 import warnings
 import collections
+from dendropy.utility import textprocessing
 from dendropy.dataio import ioservice
 from dendropy.dataio import newick
 from dendropy.dataio import nexusprocessing
@@ -45,24 +42,24 @@ class NexusWriter(ioservice.DataWriter):
 
         Keyword Arguments
         -----------------
-        simple : boolean, default: `False`
-            If `True`, write in simple NEXUS format, i.e. in a single "DATA"
+        simple : boolean, default: |False|
+            If |True|, write in simple NEXUS format, i.e. in a single "DATA"
             block, instead of separate "TAXA" and "CHARACTER" blocks.
-        suppress_taxa_blocks: boolean, default: `False`
-            If `True`, do not write a "TAXA" block. Note that this may make the
+        suppress_taxa_blocks: boolean, default: |False|
+            If |True|, do not write a "TAXA" block. Note that this may make the
             file impossible to parse if there are multiple taxon namespaces in
             the data.
-        suppress_unreferenced_taxon_namespaces: boolean, default: `False`
-            If `True`, then when writing |DataSet| objects, any
+        suppress_unreferenced_taxon_namespaces: boolean, default: |False|
+            If |True|, then when writing |DataSet| objects, any
             |TaxonNamespace| object in the DataSet's ``taxon_namespaces``
             collection will *not* be written as a "TAXA" block if it is not
             referenced by any character matrix (``char_matrices``) or tree list
             (``tree_lists``).
-        suppress_block_titles : bool or `None`
-            If `True` then 'TITLE' element to blocks will not be written. Note
+        suppress_block_titles : bool or |None|
+            If |True| then 'TITLE' element to blocks will not be written. Note
             that this may make the file impossible to parse if there are
-            multiple taxon namespaces in the data. If `False`, then the
-            'TITLE' element will always be written. Default is `None`: the
+            multiple taxon namespaces in the data. If |False|, then the
+            'TITLE' element will always be written. Default is |None|: the
             'TITLE' element will only be written if needed because there is
             more than on taxon namespace in the data.
         file_comments: iterable [``str``]
@@ -74,8 +71,8 @@ class NexusWriter(ioservice.DataWriter):
             List of strings to be written after data (e.g., PAUP blocks,
             MrBayes blocks etc.).
         allow_multiline_comments : bool
-            If `False` then comments will be merged into a single string before
-            being written. Default is `True`: each comment element will be
+            If |False| then comments will be merged into a single string before
+            being written. Default is |True|: each comment element will be
             written on its own line.
         continuous_character_state_value_format_fn : function object
             When writing |ContinuousCharacterMatrix| data: a function that
@@ -86,77 +83,77 @@ class NexusWriter(ioservice.DataWriter):
             |StandardCharacterMatrix|): a function that takes a
             standard character state value (i.e., a |StateIdentity| instance)
             and returns the string representation of it.
-        suppress_leaf_taxon_labels : boolean, default: `False`
-            If `True`, then taxon labels will not be rendered for leaves.
-            Default is `False`: render leaf taxon labels. See notes below for
+        suppress_leaf_taxon_labels : boolean, default: |False|
+            If |True|, then taxon labels will not be rendered for leaves.
+            Default is |False|: render leaf taxon labels. See notes below for
             details.
-        suppress_leaf_node_labels : boolean, default: `True`
-            If `False`, then node labels (if available) will be printed for
-            leaves. Defaults to `True`: do not render leaf node labels. See
+        suppress_leaf_node_labels : boolean, default: |True|
+            If |False|, then node labels (if available) will be printed for
+            leaves. Defaults to |True|: do not render leaf node labels. See
             notes below for details.
-        suppress_internal_taxon_labels : boolean, default: `False`
-            If `True`, then taxon labels will not be printed for internal
-            nodes. Default is `False`: print taxon labels for internal nodes.
+        suppress_internal_taxon_labels : boolean, default: |False|
+            If |True|, then taxon labels will not be printed for internal
+            nodes. Default is |False|: print taxon labels for internal nodes.
             See notes below for details.
-        suppress_internal_node_labels : boolean, default: `False`
-            If `True`, then node labels will not be printed for internal nodes.
-            Default is `False`: print node labels for internal nodes. See notes
+        suppress_internal_node_labels : boolean, default: |False|
+            If |True|, then node labels will not be printed for internal nodes.
+            Default is |False|: print node labels for internal nodes. See notes
             below for details.
-        suppress_rooting : boolean, default: `False`
-            If `True`, will not write rooting token ('[&R]' or '[&U]').
-            Default is `False`: rooting token will be written.
-        suppress_edge_lengths : boolean, default: `False`
-            If `True`, will not write edge lengths. Default is `False`: edge
+        suppress_rooting : boolean, default: |False|
+            If |True|, will not write rooting token ('[&R]' or '[&U]').
+            Default is |False|: rooting token will be written.
+        suppress_edge_lengths : boolean, default: |False|
+            If |True|, will not write edge lengths. Default is |False|: edge
             lengths will be written.
-        unquoted_underscores : boolean, default: `False`
-            If `True`, labels with underscores will not be quoted, which will
+        unquoted_underscores : boolean, default: |False|
+            If |True|, labels with underscores will not be quoted, which will
             mean that they will be interpreted as spaces if read again ("soft"
-            underscores).  If `False`, then labels with underscores
+            underscores).  If |False|, then labels with underscores
             will be quoted, resulting in "hard" underscores.  Default is
-            `False`.
-        preserve_spaces : boolean, default: `False`
-            If `True`, spaces will not be replaced with underscores in labels
+            |False|.
+        preserve_spaces : boolean, default: |False|
+            If |True|, spaces will not be replaced with underscores in labels
             (which means any labels containing spaces will have to be quoted).
-            Default is `False`: spaces will be converted to underscores.
+            Default is |False|: spaces will be converted to underscores.
             False.
-        store_tree_weights : boolean, default: `False`
-            If `True`, tree weights are written. Default is `False`: tree
+        store_tree_weights : boolean, default: |False|
+            If |True|, tree weights are written. Default is |False|: tree
             weights will not be written.
-        translate_tree_taxa : boolean or dict or `None`, default: `None`.
-            If `False` or `None`, then a "TRANSLATE" statement will not be
+        translate_tree_taxa : boolean or dict or |None|, default: |None|.
+            If |False| or |None|, then a "TRANSLATE" statement will not be
             used, and tree statements will contain the full taxon labels. If
-            not `False` or `None`, a "TRANSLATE" statement will be written and
+            not |False| or |None|, a "TRANSLATE" statement will be written and
             referenced in tree statements (instead of using the taxon labels).
-            If `True`, then a default translate statement will be used, with
+            If |True|, then a default translate statement will be used, with
             tokens given by the taxon indexes. If a dictionary is given, then
             the keys should be |Taxon| objects and the values should be the
             token (strings).
-        suppress_annotations : boolean, default: `False`
-            If `True`, metadata annotations will be ignored.
-            Defaults to `False`: metadata annotations will be written.
-        annotations_as_nhx : boolean, default: `False`
-            If `True`, and if ``suppress_annotations`` is `False`, will write
-            annotations as NHX statements. Default is `False`: annotations
+        suppress_annotations : boolean, default: |False|
+            If |True|, metadata annotations will be ignored.
+            Defaults to |False|: metadata annotations will be written.
+        annotations_as_nhx : boolean, default: |False|
+            If |True|, and if ``suppress_annotations`` is |False|, will write
+            annotations as NHX statements. Default is |False|: annotations
             will not be written as NHX statements.
-        suppress_item_comments : boolean, default: `False`
-            If `True`: comments will be ignored. Default is `False`: any
+        suppress_item_comments : boolean, default: |False|
+            If |True|: comments will be ignored. Default is |False|: any
             additional comments associated with trees, nodes, edges, etc. will
             be written.
         node_label_element_separator : string, default: ' '
             If both ``suppress_leaf_taxon_labels`` and
-            ``suppress_leaf_node_labels`` are `False`, then this will be the
+            ``suppress_leaf_node_labels`` are |False|, then this will be the
             string used to join them. Defaults to ' ' (space).
-        node_label_compose_fn : function object or `None`, default: `None`
-            If not `None`, should be a function that takes a |Node|
+        node_label_compose_fn : function object or |None|, default: |None|
+            If not |None|, should be a function that takes a |Node|
             object as an argument and returns the string to be used to
             represent the node in the tree statement. The return value from
             this function is used unconditionally to print a node
             representation in a tree statement, by-passing the default
             labelling function, ignoring ``suppress_leaf_taxon_labels``,
             ``suppress_leaf_node_labels=True``, ``suppress_internal_taxon_labels``,
-            ``suppress_internal_node_labels``, etc. Defaults to `None`.
-        edge_label_compose_fn : function object or `None`, default: `None`
-            If not `None`, should be a function that takes an Edge object as
+            ``suppress_internal_node_labels``, etc. Defaults to |None|.
+        edge_label_compose_fn : function object or |None|, default: |None|
+            If not |None|, should be a function that takes an Edge object as
             an argument, and returns the string to be used to represent the
             edge length in the tree statement.
         real_value_format_specifier : string, default: ''
@@ -165,9 +162,12 @@ class NexusWriter(ioservice.DataWriter):
             annotations. The format specifier should be given in Python's
             string format specification mini-language. E.g. ".8f", ".4E",
             "8.4f".
-        ignore_unrecognized_keyword_arguments : boolean, default: `False`
-            If `True`, then unsupported or unrecognized keyword arguments will
-            not result in an error. Default is `False`: unsupported keyword
+        exclude_from_taxa_blocks : set (or other iterable) of |Taxon| instances
+            If specified, any |Taxon| object in this set will be excluded from
+            all TAXA blocks, *even* if they are referenced in the data.
+        ignore_unrecognized_keyword_arguments : boolean, default: |False|
+            If |True|, then unsupported or unrecognized keyword arguments will
+            not result in an error. Default is |False|: unsupported keyword
             arguments will result in an error.
 
         """
@@ -198,6 +198,7 @@ class NexusWriter(ioservice.DataWriter):
         if self.discrete_character_state_value_format_fn is None:
             self.discrete_character_state_value_format_fn = self._format_discrete_character_value
         self.translate_tree_taxa = kwargs.pop("translate_tree_taxa", None)
+        self.exclude_from_taxa_blocks = kwargs.pop("exclude_from_taxa_blocks", None)
 
         # The following are used by NewickWriter in addition to NexusWriter, so
         # they are extracted/set here and then forwarded on ...
@@ -293,14 +294,21 @@ class NexusWriter(ioservice.DataWriter):
                 stream.write(block)
                 stream.write("\n")
 
+    def _get_taxa_to_include(self, taxon_namespace):
+        if not self.exclude_from_taxa_blocks:
+            return list(taxon_namespace)
+        else:
+            return [t for t in taxon_namespace if t not in self.exclude_from_taxa_blocks]
+
     def _write_taxa_block(self, stream, taxon_namespace):
         stream.write("BEGIN TAXA;\n")
         self._write_block_title(stream, taxon_namespace)
         self._write_item_annotations(stream, taxon_namespace)
         self._write_item_comments(stream, taxon_namespace)
-        stream.write("    DIMENSIONS NTAX={};\n".format(len(taxon_namespace)))
+        taxon_to_include = self._get_taxa_to_include(taxon_namespace)
+        stream.write("    DIMENSIONS NTAX={};\n".format(len(taxon_to_include)))
         stream.write("    TAXLABELS\n")
-        for taxon in taxon_namespace:
+        for taxon in taxon_to_include:
             stream.write("        {}\n".format(
                 nexusprocessing.escape_nexus_token(taxon.label, preserve_spaces=self.preserve_spaces, quote_underscores=not self.unquoted_underscores),
                 ))
@@ -433,13 +441,13 @@ class NexusWriter(ioservice.DataWriter):
 
     def _write_comments(self, stream, comments):
         if self.allow_multiline_comments:
-            if isinstance(comments, str):
+            if textprocessing.is_str_type(comments):
                 stream.write("[{}]\n".format(comments))
             else:
                 comments = "\n".join([str(c) for c in comments])
                 stream.write("[\n{}\n]\n".format(comments))
         else:
-            if isinstance(comments, str):
+            if textprocessing.is_str_type(comments):
                 comments = comments.replace("\r\n", "\n").replace("\n\r","\n").replace("\r","\n")
                 # comments = re.split(r'[\r\n]+', comments)
                 comments = [c for c in re.split(r'[\r\n]+', comments) if c]
