@@ -23,10 +23,8 @@ Character and character-sequence data structures.
 import warnings
 import copy
 import collections
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
+from dendropy.utility.textprocessing import StringIO
+from dendropy.utility import textprocessing
 from dendropy.utility import error
 from dendropy.utility import deprecate
 from dendropy.utility import container
@@ -126,7 +124,7 @@ class CharacterDataSequence(
     ``character_value`` argument, thus allowing for setting of the value,
     character type, and annotation set simultaneously. While iteration over
     character values are available through the standard list iteration
-    interface, the method ``CharacterDataSequence.iter_cells()`` provides for
+    interface, the method ``CharacterDataSequence.cell_iter()`` provides for
     iterating over ``<character-value, character-type,
     character-annotation-set>`` triplets.
 
@@ -257,7 +255,7 @@ class CharacterDataSequence(
 
     next = __next__ # Python 2 legacy support
 
-    def iter_cells(self):
+    def cell_iter(self):
         """
         Iterate over triplets of character values and associated
         |CharacterType| and |AnnotationSet| instances.
@@ -366,7 +364,7 @@ class CharacterDataSequence(
 
     def has_annotations_at(self, idx):
         """
-        Return `True` if character at ``idx`` has metadata annotations.
+        Return |True| if character at ``idx`` has metadata annotations.
 
         Parameters
         ----------
@@ -376,7 +374,7 @@ class CharacterDataSequence(
         Returns
         -------
         b : bool
-            `True` if character at ``idx`` has metadata annotations, `False`
+            |True| if character at ``idx`` has metadata annotations, |False|
             otherwise.
         """
         return not self._character_annotations[idx] is None
@@ -563,16 +561,16 @@ class CharacterMatrix(
 
             - **label** (*str*) -- Name or identifier to be assigned to the new
               object; if not given, will be assigned the one specified in the
-              data source, or `None` otherwise.
+              data source, or |None| otherwise.
             - **taxon_namespace** (|TaxonNamespace|) -- The |TaxonNamespace|
               instance to use to :doc:`manage the taxon names </primer/taxa>`.
               If not specified, a new one will be created.
             - **matrix_offset** (*int*) -- 0-based index of character block or
               matrix in source to be parsed. If not specified then the
               first matrix (offset = 0) is assumed.
-            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If `True`,
+            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If |True|,
               then unsupported or unrecognized keyword arguments will not
-              result in an error. Default is `False`: unsupported keyword
+              result in an error. Default is |False|: unsupported keyword
               arguments will result in an error.
 
         **Optional Schema-Specific Keyword Arguments:**
@@ -700,7 +698,7 @@ class CharacterMatrix(
         values are of type `CharacterDataSequence`, then they are added
         as-is.  Otherwise `CharacterDataSequence` instances are
         created for them. Values may be coerced into types compatible with
-        particular matrices. The classmethod `coerce_values()` will be
+        particular matrices. The classmethod ``coerce_values()`` will be
         called for this.
 
         Examples
@@ -732,9 +730,9 @@ class CharacterMatrix(
             specified, a new one will be created using keyword arguments
             specified by ``kwargs``.
         case_sensitive_taxon_labels : boolean
-            If `True`, matching of string labels specified as keys in ``d`` will
+            If |True|, matching of string labels specified as keys in ``d`` will
             be matched to |Taxon| objects in current taxon namespace
-            with case being respected. If `False`, then case will be ignored.
+            with case being respected. If |False|, then case will be ignored.
         \*\*kwargs : keyword arguments, optional
             Keyword arguments to be passed to constructor of
             |CharacterMatrix| when creating new instance to populate, if
@@ -748,46 +746,18 @@ class CharacterMatrix(
         if char_matrix is None:
             char_matrix = cls(**kwargs)
         for key in source_dict:
-            if isinstance(key, str):
+            if textprocessing.is_str_type(key):
                 taxon = char_matrix.taxon_namespace.require_taxon(key,
                         is_case_sensitive=case_sensitive_taxon_labels)
             else:
                 taxon = key
                 if taxon not in char_matrix.taxon_namespace:
                     char_matrix.taxon_namespace.add_taxon(taxon)
-            s = cls.coerce_values(source_dict[key])
+            s = char_matrix.coerce_values(source_dict[key])
             char_matrix[taxon] = s
         return char_matrix
     from_dict = classmethod(from_dict)
 
-    def coerce_values(cls, values):
-        """
-        Converts elements of ``values`` to type of matrix.
-
-        This method is called by :meth:`CharacterMatrix.from_dict` to create
-        sequences from iterables of values.  This method should be overridden
-        by derived classes to ensure that ``values`` consists of types compatible
-        with the particular type of matrix. For example, a CharacterMatrix type
-        with a fixed state alphabet (such as |DnaCharacterMatrix|) would
-        dereference the string elements of ``values`` to return a list of
-        |StateIdentity| objects corresponding to the symbols represented
-        by the strings.  If there is no value-type conversion done, then
-        ``values`` should be returned as-is. If no value-type conversion is
-        possible (e.g., when the type of a value is dependent on positionaly
-        information), then a TypeError should be raised.
-
-        Parameters
-        ----------
-        values : iterable
-            Iterable of values to be converted.
-
-        Returns
-        -------
-        v : list of values.
-        """
-        return values
-    coerce_values = classmethod(coerce_values)
-
     ###########################################################################
     ### Lifecycle and Identity
 
@@ -990,7 +960,7 @@ class CharacterMatrix(
                 taxon = self.taxon_namespace[key]
             else:
                 raise IndexError(key)
-        elif isinstance(key, str):
+        elif textprocessing.is_str_type(key):
             taxon = self.taxon_namespace.get_taxon(label=key)
             if taxon is None:
                 raise KeyError(key)
@@ -1007,7 +977,7 @@ class CharacterMatrix(
         ----------
         taxon : |Taxon|
             |Taxon| instance with which this sequence is associated.
-        values : iterable or `None`
+        values : iterable or |None|
             An initial set of values with which to populate the new character
             sequence.
 
@@ -1089,8 +1059,8 @@ class CharacterMatrix(
 
     def __contains__(self, key):
         """
-        Returns `True` if a sequence associated with ``key`` is in ``self``, or
-        `False` otherwise.
+        Returns |True| if a sequence associated with ``key`` is in ``self``, or
+        |False| otherwise.
 
         Parameters
         ----------
@@ -1106,7 +1076,7 @@ class CharacterMatrix(
         Returns
         -------
         b : boolean
-            `True` if ``key`` is in ``self``; `False` otherwise.
+            |True| if ``key`` is in ``self``; |False| otherwise.
         """
         return self._taxon_sequence_map.__contains__(key)
 
@@ -1153,6 +1123,36 @@ class CharacterMatrix(
                 message="Deprecated since DendroPy 4: 'vectors()' will no longer be supported in future releases; use 'sequences()' instead")
         return self.sequences()
 
+    ###########################################################################
+    ### Symbol/alphabet management
+
+    def coerce_values(self, values):
+        """
+        Converts elements of ``values`` to type of matrix.
+
+        This method is called by :meth:`CharacterMatrix.from_dict` to create
+        sequences from iterables of values.  This method should be overridden
+        by derived classes to ensure that ``values`` consists of types compatible
+        with the particular type of matrix. For example, a CharacterMatrix type
+        with a fixed state alphabet (such as |DnaCharacterMatrix|) would
+        dereference the string elements of ``values`` to return a list of
+        |StateIdentity| objects corresponding to the symbols represented
+        by the strings.  If there is no value-type conversion done, then
+        ``values`` should be returned as-is. If no value-type conversion is
+        possible (e.g., when the type of a value is dependent on positionaly
+        information), then a TypeError should be raised.
+
+        Parameters
+        ----------
+        values : iterable
+            Iterable of values to be converted.
+
+        Returns
+        -------
+        v : list of values.
+        """
+        return values
+
     ###########################################################################
     ### Sequence Access Iteration
 
@@ -1265,11 +1265,11 @@ class CharacterMatrix(
             A valid value (e.g., a numeric value for continuous characters, or
             a |StateIdentity| for discrete character).
         size : integer or None
-            The size (length) up to which the sequences will be padded. If `None`, then
+            The size (length) up to which the sequences will be padded. If |None|, then
             the maximum (longest) sequence size will be used.
         append : boolean
-            If `True` (default), then new values will be added to the end of
-            each sequence. If `False`, then new values will be inserted to the
+            If |True| (default), then new values will be added to the end of
+            each sequence. If |False|, then new values will be inserted to the
             front of each sequence.
         """
         if size is None:
@@ -1307,11 +1307,11 @@ class CharacterMatrix(
             A valid value (e.g., a numeric value for continuous characters, or
             a |StateIdentity| for discrete character).
         size : integer or None
-            The size (length) up to which the sequences will be padded. If `None`, then
+            The size (length) up to which the sequences will be padded. If |None|, then
             the maximum (longest) sequence size will be used.
         append : boolean
-            If `True` (default), then new values will be added to the end of
-            each sequence. If `False`, then new values will be inserted to the
+            If |True| (default), then new values will be added to the end of
+            each sequence. If |False|, then new values will be inserted to the
             front of each sequence.
         """
         self.fill_taxa()
@@ -1588,7 +1588,7 @@ class CharacterMatrix(
         of columns given by the CharacterSubset, ``character_subset``.
         Note that this new matrix will still reference the same taxon set.
         """
-        if isinstance(character_subset, str):
+        if textprocessing.is_str_type(character_subset):
             if character_subset not in self.character_subsets:
                 raise KeyError(character_subset)
             else:
@@ -1605,7 +1605,7 @@ class CharacterMatrix(
         # clear out character subsets; otherwise all indices will have to be
         # recalculated, which will require some careful and perhaps arbitrary
         # handling of corner cases
-        clone.character_subsets = []
+        clone.character_subsets = container.OrderedCaselessDict()
         # clone.clone_from(self)
         for vec in clone.values():
             for cell_idx in range(len(vec)-1, -1, -1):
@@ -1665,6 +1665,31 @@ class CharacterMatrix(
 ###############################################################################
 ## Specialized Matrices
 
+### Continuous Characters ##################################################
+
+class ContinuousCharacterDataSequence(CharacterDataSequence):
+    """
+    A sequence of continuous character values for a particular taxon or entry
+    in a data matrix. Specializes `CharacterDataSequence` by assuming all
+    values are primitive numerics (i.e., either floats or integers) when
+    copying or representing self.
+    """
+
+    def symbols_as_list(self):
+        """
+        Returns list of string representation of values of this vector.
+
+        Returns
+        -------
+        v : list
+            List of string representation of values making up this vector.
+        """
+        return [str(v) for v in self]
+
+    def symbols_as_string(self, sep=" "):
+        # different default
+        return CharacterDataSequence.symbols_as_string(self, sep=sep)
+
 class ContinuousCharacterMatrix(CharacterMatrix):
     """
     Specializes |CharacterMatrix| for continuous data.
@@ -1673,39 +1698,19 @@ class ContinuousCharacterMatrix(CharacterMatrix):
     elements assumed to be ``float`` .
     """
 
-    class ContinuousCharacterDataSequence(CharacterDataSequence):
-        """
-        A sequence of continuous character values for a particular taxon or entry
-        in a data matrix. Specializes `CharacterDataSequence` by assuming all
-        values are primitive numerics (i.e., either floats or integers) when
-        copying or representing self.
-        """
-
-        def symbols_as_list(self):
-            """
-            Returns list of string representation of values of this vector.
-
-            Returns
-            -------
-            v : list
-                List of string representation of values making up this vector.
-            """
-            return [str(v) for v in self]
-
-        def symbols_as_string(self, sep=" "):
-            # different default
-            return CharacterDataSequence.symbols_as_string(self, sep=sep)
-
     character_sequence_type = ContinuousCharacterDataSequence
     data_type = "continuous"
 
     def __init__(self, *args, **kwargs):
         CharacterMatrix.__init__(self, *args, **kwargs)
 
+### Discrete Characters ##################################################
+
+class DiscreteCharacterDataSequence(CharacterDataSequence):
+    pass
+
 class DiscreteCharacterMatrix(CharacterMatrix):
 
-    class DiscreteCharacterDataSequence(CharacterDataSequence):
-        pass
     character_sequence_type = DiscreteCharacterDataSequence
 
     data_type = "discrete"
@@ -1735,11 +1740,11 @@ class DiscreteCharacterMatrix(CharacterMatrix):
         if taxon not in self:
             self[taxon] = CharacterDataSequence()
         for value in state_symbols:
-            if isinstance(value, str):
+            if textprocessing.is_str_type(value):
                 symbol = value
             else:
                 symbol = str(value)
-            self[taxon].append(CharacterDataCell(value=self.default_symbol_state_map[symbol]))
+            self[taxon].append(self.default_symbol_state_map[symbol])
 
     def remap_to_state_alphabet_by_symbol(self,
             state_alphabet,
@@ -1751,14 +1756,13 @@ class DiscreteCharacterMatrix(CharacterMatrix):
         reassigned to any state alphabet element in ``sa`` that has the same
         symbol. Raises KeyError if no matching symbol can be found.
         """
-        symbol_state_map = state_alphabet.symbol_state_map()
         for vi, vec in enumerate(self._taxon_sequence_map.values()):
             for ci, cell in enumerate(vec):
-                cell.value = symbol_state_map[cell.value.symbol]
+                vec[ci] = state_alphabet[cell.symbol]
         for ct in self.character_types:
-            ct.state_alphabet = state_alphabet
+            if ct is not None:
+                ct.state_alphabet = state_alphabet
         if purge_other_state_alphabets:
-            self.state_alphabets = [state_alphabet]
             self.default_state_alphabet = state_alphabet
 
     def remap_to_default_state_alphabet_by_symbol(self,
@@ -1789,11 +1793,11 @@ class DiscreteCharacterMatrix(CharacterMatrix):
 
         char_indices : iterable of ints
             An iterable of indexes of characters to include (by column). If not
-            given or `None` [default], then all characters are included.
+            given or |None| [default], then all characters are included.
 
         gaps_as_missing : boolean
-            If `True` [default] then gap characters will be treated as missing
-            data values. If `False`, then they will be treated as an additional
+            If |True| [default] then gap characters will be treated as missing
+            data values. If |False|, then they will be treated as an additional
             (fundamental) state.`
 
         Returns
@@ -1851,10 +1855,13 @@ class DiscreteCharacterMatrix(CharacterMatrix):
             taxon_to_state_indices[t] = v
         return taxon_to_state_indices
 
+### Fixed Alphabet Characters ##################################################
+
+class FixedAlphabetCharacterDataSequence(CharacterDataSequence):
+    pass
+
 class FixedAlphabetCharacterMatrix(DiscreteCharacterMatrix):
 
-    class FixedAlphabetCharacterDataSequence(CharacterDataSequence):
-        pass
     character_sequence_type = FixedAlphabetCharacterDataSequence
     data_type = "fixed"
     datatype_alphabet = None
@@ -1864,74 +1871,102 @@ class FixedAlphabetCharacterMatrix(DiscreteCharacterMatrix):
         self.state_alphabets.append(self.__class__.datatype_alphabet)
         self._default_state_alphabet = self.__class__.datatype_alphabet
 
+    def coerce_values(self, values):
+        if self.datatype_alphabet is None:
+            raise ValueError("'datatype_alphabet' not set")
+        return charstatemodel.coerce_to_state_identities(
+                state_alphabet=self.datatype_alphabet,
+                values=values)
+
+### DNA Characters ##################################################
+
+class DnaCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class DnaCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for DNA data.
     """
-    class DnaCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = DnaCharacterDataSequence
     data_type = "dna"
     datatype_alphabet = DNA_STATE_ALPHABET
 
+### RNA Characters ##################################################
+
+class RnaCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class RnaCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for DNA data.
     """
-    class RnaCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = RnaCharacterDataSequence
     data_type = "rna"
     datatype_alphabet = RNA_STATE_ALPHABET
 
+### Nucleotide Characters ##################################################
+
+class NucleotideCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class NucleotideCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for RNA data.
     """
-    class NucleotideCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = NucleotideCharacterDataSequence
     data_type = "nucleotide"
     datatype_alphabet = NUCLEOTIDE_STATE_ALPHABET
 
+### Protein Characters ##################################################
+
+class ProteinCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class ProteinCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for protein or amino acid data.
     """
-    class ProteinCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = ProteinCharacterDataSequence
     data_type = "protein"
     datatype_alphabet = PROTEIN_STATE_ALPHABET
 
+### Restricted Site Characters ##################################################
+
+class RestrictionSitesCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class RestrictionSitesCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for restriction site data.
     """
-    class RestrictionSitesCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = RestrictionSitesCharacterDataSequence
     data_type = "restriction"
     datatype_alphabet = RESTRICTION_SITES_STATE_ALPHABET
 
+### Infinite Sites Characters ##################################################
+
+class InfiniteSitesCharacterDataSequence(FixedAlphabetCharacterDataSequence):
+    pass
+
 class InfiniteSitesCharacterMatrix(FixedAlphabetCharacterMatrix):
     """
     Specializes |CharacterMatrix| for infinite sites data.
     """
-    class InfiniteSitesCharacterDataSequence(FixedAlphabetCharacterMatrix.FixedAlphabetCharacterDataSequence):
-        pass
     character_sequence_type = InfiniteSitesCharacterDataSequence
     data_type = "infinite"
     datatype_alphabet = INFINITE_SITES_STATE_ALPHABET
 
+### Standard Characters ##################################################
+
+class StandardCharacterDataSequence(DiscreteCharacterDataSequence):
+    pass
+
 class StandardCharacterMatrix(DiscreteCharacterMatrix):
     """
     Specializes |CharacterMatrix| for "standard" data (i.e., generic discrete
     character data).
 
     """
-    class StandardCharacterDataSequence(DiscreteCharacterMatrix.DiscreteCharacterDataSequence):
-        pass
     character_sequence_type = StandardCharacterDataSequence
 
     data_type = "standard"
@@ -1954,6 +1989,14 @@ class StandardCharacterMatrix(DiscreteCharacterMatrix):
         if default_state_alphabet is not None:
             self.default_state_alphabet = default_state_alphabet
 
+    def coerce_values(self, values):
+        if self.default_state_alphabet is None:
+            raise ValueError("'default_state_alphabet' not set")
+        return charstatemodel.coerce_to_state_identities(
+                state_alphabet=self.default_state_alphabet,
+                values=values)
+
+
 ###############################################################################
 ## Main Character Matrix Factory Function
 
@@ -1961,6 +2004,7 @@ data_type_matrix_map = {
     'continuous' : ContinuousCharacterMatrix,
     'dna' : DnaCharacterMatrix,
     'rna' : RnaCharacterMatrix,
+    'nucleotide' : NucleotideCharacterMatrix,
     'protein' : ProteinCharacterMatrix,
     'standard' : StandardCharacterMatrix,
     'restriction' : RestrictionSitesCharacterMatrix,
