@@ -930,5 +930,70 @@ class NewickTreeReaderOffsetTreeTest(
     #             with self.assertRaises(IndexError):
     #                 f(src, "newick", collection_offset=1, tree_offset=0)
 
+class JplaceParsingTest(dendropytest.ExtendedTestCase):
+
+    def test_basic(self):
+        s = "((A:.01[e]{0}, B:.02{1})D:.3{3}[g], C:.04{4}[h]) {5};"
+        tree = dendropy.Tree.get_from_string(s,
+                                             "newick",
+                                             is_parse_jplace_tokens=True)
+        self.assertEqual([0.01, 0.02, 0.3, 0.04, None],
+                         [e.length for e in tree.edge_index])
+        self.assertEqual('((A:0.01,B:0.02)D:0.3,C:0.04)',
+                         str(tree))
+        self.assertEqual([0,1,3,4,5],
+                         [e.edge_number for e in tree.edge_index])
+
+    def test_do_not_parse_by_default(self):
+        s = "((A:.01[e]{0}, B:.02{1})D:.3{3}[g], C:.04{4}[h]) {5};"
+        with self.assertRaises(newickreader.NewickReader.NewickReaderMalformedStatementError):
+            try:
+                tree = dendropy.Tree.get_from_string(s, "newick")
+            except newickreader.NewickReader.NewickReaderMalformedStatementError as e:
+                self.assertEqual(
+                    "Expecting ':', ')', ',' or ';' after reading label but found '{'",
+                    e.message)
+                raise e
+
+    def test_malformed_error_message_parsing_jplace(self):
+        s = "((A}:.01[e]{0}, B:.02{1})D:.3{3}[g], C:.04{4}[h]) {5};"
+        with self.assertRaises(newickreader.NewickReader.NewickReaderMalformedStatementError):
+            try:
+                tree = dendropy.Tree.get_from_string(s, "newick", is_parse_jplace_tokens=True)
+            except newickreader.NewickReader.NewickReaderMalformedStatementError as e:
+                self.assertEqual(
+                    # extra 2nd element of list
+                    "Expecting ':', '{', ')', ',' or ';' after reading label but found '}'",
+                    e.message)
+                raise e
+
+class NewickInternalLabelAssociationTest(unittest.TestCase):
+
+    def test_assign_to_edges(self):
+        s = "((C:1.3,D:4.0)34:0.034,(A:1.1,(B:1.2,X:1.6)26:0.026)12:0.0126,E:1.5)seed;"
+        expected_labels = {
+                0 :"seed",
+                60 :"34",
+                28 :"12",
+                24 :"26",
+                }
+        for labels_to_edges in (True, False):
+            for rooting in ("force-rooted", "force-unrooted"):
+                tree = dendropy.Tree.get(
+                        data=s,
+                        schema="newick",
+                        is_assign_internal_labels_to_edges=labels_to_edges)
+                tree.encode_bipartitions()
+                for nd in tree:
+                    if nd.is_leaf():
+                        continue
+                    expected_label = expected_labels[int(nd.bipartition)]
+                    if labels_to_edges:
+                        self.assertIs(nd.label, None)
+                        self.assertEquals(nd.edge.label, expected_label)
+                    else:
+                        self.assertEquals(nd.label, expected_label)
+                        self.assertIs(nd.edge.label, None)
+
 if __name__ == "__main__":
     unittest.main()
